@@ -32,13 +32,11 @@
 #include <stddef.h>
 
 #include <afs/opr.h>
-#include <opr/lock.h>
 #include <afs/afsint.h>
-#include <rx/rx_queue.h>
-
 #include "nfs.h"
 #include <afs/errors.h>
 #include "salvsync.h"
+#include "lwp.h"
 #include "lock.h"
 #include <afs/afssyscalls.h>
 #include "ihandle.h"
@@ -186,7 +184,7 @@ LookupNode(afs_uint32 vid, char * partName,
 	   struct SalvageQueueNode ** parent)
 {
     struct rx_queue *qp, *nqp;
-    struct SalvageQueueNode *vsp = NULL;
+    struct SalvageQueueNode *vsp;
     int idx = VSHASH(vid);
 
     for (queue_Scan(&SalvageHashTable[idx], qp, nqp, rx_queue)) {
@@ -275,10 +273,9 @@ SALVSYNC_salvInit(void)
     }
 
     /* start the salvsync thread */
-    opr_Verify(pthread_attr_init(&tattr) == 0);
-    opr_Verify(pthread_attr_setdetachstate(&tattr,
-					   PTHREAD_CREATE_DETACHED) == 0);
-    opr_Verify(pthread_create(&tid, &tattr, SALVSYNC_syncThread, NULL) == 0);
+    osi_Assert(pthread_attr_init(&tattr) == 0);
+    osi_Assert(pthread_attr_setdetachstate(&tattr, PTHREAD_CREATE_DETACHED) == 0);
+    osi_Assert(pthread_create(&tid, &tattr, SALVSYNC_syncThread, NULL) == 0);
 }
 
 static void
@@ -308,7 +305,7 @@ SALVSYNC_syncThread(void * args)
     /* when we fork, the child needs to close the salvsync server sockets,
      * otherwise, it may get salvsync requests, instead of the parent
      * salvageserver */
-    opr_Verify(pthread_atfork(NULL, NULL, CleanFDs) == 0);
+    osi_Assert(pthread_atfork(NULL, NULL, CleanFDs) == 0);
 
     SYNC_getAddr(&state->endpoint, &state->addr);
     SYNC_cleanupSock(state);
@@ -319,7 +316,7 @@ SALVSYNC_syncThread(void * args)
 
     state->fd = SYNC_getSock(&state->endpoint);
     code = SYNC_bindSock(state);
-    opr_Assert(!code);
+    osi_Assert(!code);
 
     InitHandler();
     AcceptOn();
@@ -357,7 +354,7 @@ SALVSYNC_newconnection(int afd)
 	osi_Panic("SALVSYNC_newconnection:  accept failed, errno==%d\n", errno);
     } else if (!AddHandler(fd, SALVSYNC_com)) {
 	AcceptOff();
-	opr_Verify(AddHandler(fd, SALVSYNC_com));
+	osi_Assert(AddHandler(fd, SALVSYNC_com));
     }
 }
 
@@ -759,8 +756,7 @@ static void
 AcceptOn(void)
 {
     if (AcceptHandler == -1) {
-	opr_Verify(AddHandler(salvsync_server_state.fd,
-			      SALVSYNC_newconnection));
+	osi_Assert(AddHandler(salvsync_server_state.fd, SALVSYNC_newconnection));
 	AcceptHandler = FindHandler(salvsync_server_state.fd);
     }
 }
@@ -769,7 +765,7 @@ static void
 AcceptOff(void)
 {
     if (AcceptHandler != -1) {
-	opr_Verify(RemoveHandler(salvsync_server_state.fd));
+	osi_Assert(RemoveHandler(salvsync_server_state.fd));
 	AcceptHandler = -1;
     }
 }
@@ -894,12 +890,14 @@ AllocNode(struct SalvageQueueNode ** node_out)
     int code = 0;
     struct SalvageQueueNode * node;
 
-    *node_out = node = calloc(1, sizeof(struct SalvageQueueNode));
+    *node_out = node = (struct SalvageQueueNode *)
+	malloc(sizeof(struct SalvageQueueNode));
     if (node == NULL) {
 	code = 1;
 	goto done;
     }
 
+    memset(node, 0, sizeof(struct SalvageQueueNode));
     node->type = SALVSYNC_VOLGROUP_PARENT;
     node->state = SALVSYNC_STATE_UNKNOWN;
 
@@ -1129,7 +1127,7 @@ UpdateCommandPrio(struct SalvageQueueNode * node)
     afs_int32 id;
     afs_uint32 prio;
 
-    opr_Assert(queue_IsOnQueue(node));
+    osi_Assert(queue_IsOnQueue(node));
 
     prio = node->command.sop.prio;
     id = node->partition_id;
@@ -1221,7 +1219,7 @@ SALVSYNC_getWork(void)
     osi_Panic("Node not found\n");
 
  have_node:
-    opr_Assert(node != NULL);
+    osi_Assert(node != NULL);
     node->pid = 0;
     partition_salvaging[node->partition_id]++;
     DeleteFromSalvageQueue(node);
